@@ -13,7 +13,8 @@ from .fields import (
     EncryptedCharField, EncryptedDateField,
     EncryptedDateTimeField, EncryptedIntField,
     EncryptedLongField, EncryptedFloatField, PickleField,
-    EncryptedUSPhoneNumberField, EncryptedEmailField,
+    EncryptedUSPhoneNumberField, EncryptedUSSocialSecurityNumberField,
+    EncryptedEmailField,
 )
 
 from .models import ModelWithPrivateFields
@@ -21,7 +22,7 @@ from .models import ModelWithPrivateFields
 
 class EncObject(models.Model):
     max_password = 20
-    password = EncryptedCharField(max_length=max_password)
+    password = EncryptedCharField(max_length=max_password, null=True)
 
 
 class EncDate(models.Model):
@@ -58,6 +59,14 @@ class EmailObject(models.Model):
 class USPhoneNumberField(models.Model):
     phone = EncryptedUSPhoneNumberField()
 
+class USSocialSecurityNumberField(models.Model):
+    ssn = EncryptedUSSocialSecurityNumberField()
+
+class CipherEncObject(models.Model):
+    max_password = 20
+    password = EncryptedCharField(
+        max_length=max_password,
+        block_type = 'MODE_CBC')
 
 class EncryptTests(unittest.TestCase):
 
@@ -78,6 +87,22 @@ class EncryptTests(unittest.TestCase):
         encrypted_password = self._get_encrypted_password(obj.id)
         self.assertNotEqual(encrypted_password, password)
         self.assertTrue(encrypted_password.startswith('$AES$'))
+        
+    def test_encryption_w_cipher(self):
+        """
+        Test that the database values are actually encrypted when using
+        non-default cipher types.
+        """
+        password = 'this is a password!!'  # 20 chars
+        obj = CipherEncObject(password = password)
+        obj.save()
+        # The value from the retrieved object should be the same...
+        obj = CipherEncObject.objects.get(id=obj.id)
+        self.assertEqual(password, obj.password)
+        # ...but the value in the database should not
+        encrypted_password = self._get_encrypted_password_cipher(obj.id)
+        self.assertNotEqual(encrypted_password, password)
+        self.assertTrue(encrypted_password.startswith('$AES$MODE_CBC$'))
 
     def test_max_field_length(self):
         password = 'a' * EncObject.max_password
@@ -119,6 +144,17 @@ class EncryptTests(unittest.TestCase):
             password = 'a' * pwd_length  # 'a', 'aa', ...
             self.assertTrue(enc_field._get_padding(password) >= 2)
 
+    def test_none_value(self):
+        """
+        A value of None should be passed through without encryption.
+        """
+        obj = EncObject(password=None)
+        obj.save()
+        obj = EncObject.objects.get(id=obj.id)
+        self.assertEqual(obj.password, None)
+        encrypted_text = self._get_encrypted_password(obj.id)
+        self.assertEqual(encrypted_text, None)
+
     ### Utility methods for tests ###
 
     def _get_encrypted_password(self, id):
@@ -127,6 +163,13 @@ class EncryptTests(unittest.TestCase):
         passwords = map(lambda x: x[0], cursor.fetchall())
         self.assertEqual(len(passwords), 1)  # only one
         return passwords[0]
+    
+    def _get_encrypted_password_cipher(self, id):
+        cursor = connection.cursor()
+        cursor.execute("select password from django_fields_cipherencobject where id = %s", [id,])
+        passwords = map(lambda x: x[0], cursor.fetchall())
+        self.assertEqual(len(passwords), 1)  # only one
+        return passwords[0]
 
     def _get_two_passwords(self, pwd_length):
         password = 'a' * pwd_length  # 'a', 'aa', ...
